@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: common_sighandler.h 12124 2013-09-06 21:45:03Z fwobbe $
+ *	$Id: common_sighandler.h 17449 2017-01-16 21:27:04Z pwessel $
  *
- *	Copyright (c) 1991-2013 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,11 @@
  * Version: 5
  */
 
+/*!
+ * \file common_sighandler.h
+ * \brief Prototype of a signal-handling function
+ */
+
 #pragma once
 #ifndef _COMMON_SIGHANDLER_H
 #define _COMMON_SIGHANDLER_H
