@@ -1,6 +1,6 @@
-/*      $Id: gmt_agc_io.c 12140 2013-09-09 18:12:04Z pwessel $
+/*      $Id: gmt_agc_io.c 17449 2017-01-16 21:27:04Z pwessel $
  *
- *	Copyright (c) 1991-2013 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -31,24 +31,17 @@
  * Author :	Paul Wessel, by modifying code from Robert Helie
  * Date :	09-APR-2006
  * Purpose:	To transform to/from AGC grid file format
- * Functions :	GMT_agc_read_grd_info, GMT_agc_write_grd_info,
- *		GMT_agc_write_grd_info, GMT_agc_read_grd, GMT_agc_write_grd
+ * Functions :	gmt_agc_read_grd_info, gmt_agc_write_grd_info,
+ *		gmt_agc_write_grd_info, gmt_agc_read_grd, gmt_agc_write_grd
  *-----------------------------------------------------------*/
 
 /* Public Functions:
 
-int GMT_is_agc_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
-int GMT_agc_read_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
-int GMT_agc_write_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
-int GMT_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode)
-int GMT_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode)
-
-Private Functions used by the public functions:
-
-int ReadRecord (FILE *fpi, float z[ZBLOCKWIDTH][ZBLOCKHEIGHT])
-int WriteRecord (FILE *file, float *outz[ZBLOCKWIDTH][ZBLOCKHEIGHT], float *prerec, float *postrec)
-void packAGCheader (float *prez, float *postz, struct GMT_GRID_HEADER *header)
-void SaveAGCHeader (char *remark, float *agchead)
+int gmt_is_agc_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
+int gmt_agc_read_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
+int gmt_agc_write_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
+int gmt_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode)
+int gmt_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode)
 
 */
 
@@ -63,37 +56,39 @@ void SaveAGCHeader (char *remark, float *agchead)
 # define AGCHEADINDICATOR	"agchd:"
 # define PARAMSIZE		((GMT_GRID_REMARK_LEN160 - HEADINDSIZE) / BUFFHEADSIZE)
 
-int ReadRecord (FILE *fpi, float z[ZBLOCKWIDTH][ZBLOCKHEIGHT])
-{	/* Reads one block of data, including pre- and post-headers */
+/* Local runctions used by the public functions: */
+
+GMT_LOCAL int agc_read_record (FILE *fpi, float z[ZBLOCKWIDTH][ZBLOCKHEIGHT]) {
+	/* Reads one block of data, including pre- and post-headers */
 	size_t nitems;
 	float garbage[PREHEADSIZE];
 
-	if (GMT_fread (garbage, sizeof(float), PREHEADSIZE, fpi) < PREHEADSIZE)
+	if (gmt_M_fread (garbage, sizeof(float), PREHEADSIZE, fpi) < PREHEADSIZE)
 		return (GMT_GRDIO_READ_FAILED);
-	nitems = GMT_fread (z, sizeof(float), ZBLOCKWIDTH * ZBLOCKHEIGHT, fpi);
+	nitems = gmt_M_fread (z, sizeof(float), ZBLOCKWIDTH * ZBLOCKHEIGHT, fpi);
 
 	if (nitems != ZBLOCKWIDTH * ZBLOCKHEIGHT && !feof(fpi))
 		return (GMT_GRDIO_READ_FAILED);	/* Bad stuff */
-	if (GMT_fread (garbage, sizeof(float), POSTHEADSIZE, fpi) < POSTHEADSIZE)
+	if (gmt_M_fread (garbage, sizeof(float), POSTHEADSIZE, fpi) < POSTHEADSIZE)
 		return (GMT_GRDIO_READ_FAILED);
 	return (GMT_NOERROR);
 }
 
-int WriteRecord (FILE *file, float rec[ZBLOCKWIDTH][ZBLOCKHEIGHT], float *prerec, float *postrec)
-{	/* Writes one block of data, including pre- and post-headers */
-	if (GMT_fwrite (prerec, sizeof(float), PREHEADSIZE, file) < PREHEADSIZE)
+GMT_LOCAL int agc_write_record (FILE *file, float rec[ZBLOCKWIDTH][ZBLOCKHEIGHT], float *prerec, float *postrec) {
+	/* Writes one block of data, including pre- and post-headers */
+	if (gmt_M_fwrite (prerec, sizeof(float), PREHEADSIZE, file) < PREHEADSIZE)
 		return (GMT_GRDIO_WRITE_FAILED);
-	if (GMT_fwrite (rec, sizeof(float), ZBLOCKWIDTH * ZBLOCKHEIGHT, file) < ZBLOCKWIDTH * ZBLOCKHEIGHT)
+	if (gmt_M_fwrite (rec, sizeof(float), ZBLOCKWIDTH * ZBLOCKHEIGHT, file) < ZBLOCKWIDTH * ZBLOCKHEIGHT)
 		return (GMT_GRDIO_WRITE_FAILED);
-	if (GMT_fwrite (postrec, sizeof(float), POSTHEADSIZE, file) < POSTHEADSIZE)
+	if (gmt_M_fwrite (postrec, sizeof(float), POSTHEADSIZE, file) < POSTHEADSIZE)
 		return (GMT_GRDIO_WRITE_FAILED);
 	return (GMT_NOERROR);
 }
 
-void packAGCheader (float *prez, float *postz, struct GMT_GRID_HEADER *header)
-{	/* Places grd header info in the AGC header array */
-	GMT_memset (prez,  PREHEADSIZE,  float);
-	GMT_memset (postz, POSTHEADSIZE, float);
+GMT_LOCAL void agc_pack_header (float *prez, float *postz, struct GMT_GRID_HEADER *header) {
+	/* Places grd header info in the AGC header array */
+	gmt_M_memset (prez,  PREHEADSIZE,  float);
+	gmt_M_memset (postz, POSTHEADSIZE, float);
 	prez[0] = (float)header->wesn[YLO];
 	prez[1] = (float)header->wesn[YHI];
 	prez[2] = (float)header->wesn[XLO];
@@ -103,24 +98,29 @@ void packAGCheader (float *prez, float *postz, struct GMT_GRID_HEADER *header)
 	prez[PREHEADSIZE-1] = (float)RECORDLENGTH;
 }
 
-void SaveAGCHeader (char *remark, float *agchead)
-{	/* Place AGC header data in remark string */
+GMT_LOCAL void agc_save_header (char *remark, float *agchead) {
+	/* Place AGC header data in remark string */
 	char floatvalue[PARAMSIZE+1];	/* Allow space for final \0 */
 	unsigned int i;
 	size_t j;
 
 	strcpy (remark, AGCHEADINDICATOR);
 	for (i = 0; i < BUFFHEADSIZE; i++) {
-		sprintf (floatvalue, "%f", agchead[i]);
+		snprintf (floatvalue, PARAMSIZE, "%f", agchead[i]);
 		for (j = strlen (floatvalue); j < PARAMSIZE; j++) strcat (floatvalue, " ");
 		strcat (remark, floatvalue);
 	}
 }
 
-int GMT_is_agc_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
+/*----------------------------------------------------------|
+ * Public functions that are part of the GMT Devel library  |
+ *----------------------------------------------------------|
+ */
+
+int gmt_is_agc_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
 	/* Determine if file is an AGC grid file. */
 	FILE *fp = NULL;
-	int nx, ny;
+	int n_columns, n_rows;
 	off_t predicted_size;
 	float recdata[RECORDLENGTH], x_min, x_max, y_min, y_max, x_inc, y_inc;
 	struct stat buf;
@@ -129,28 +129,41 @@ int GMT_is_agc_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
 		return (GMT_GRDIO_PIPE_CODECHECK);		/* Cannot check on pipes */
 	if (stat (header->name, &buf))
 		return (GMT_GRDIO_STAT_FAILED);			/* Inquiry about file failed somehow */
-	if ((fp = GMT_fopen (GMT, header->name, "rb")) == NULL)
+	if ((fp = gmt_fopen (GMT, header->name, "rb")) == NULL)
 		return (GMT_GRDIO_OPEN_FAILED);			/* Opening the file failed somehow */
-	if (GMT_fread (recdata, sizeof(float), RECORDLENGTH, fp) < RECORDLENGTH)
+	if (gmt_M_fread (recdata, sizeof(float), RECORDLENGTH, fp) < RECORDLENGTH) {
+		gmt_fclose (GMT, fp);
 		return (GMT_GRDIO_READ_FAILED);
+	}
 
 	y_min = recdata[0];	y_max = recdata[1];
-	if (y_min >= y_max)
+	if (y_min >= y_max) {
+		gmt_fclose (GMT, fp);
 		return (GMT_GRDIO_BAD_VAL);
+	}
 	x_min = recdata[2];	x_max = recdata[3];
-	if (x_min >= x_max)
+	if (x_min >= x_max) {
+		gmt_fclose (GMT, fp);
 		return (GMT_GRDIO_BAD_VAL);
+	}
 	y_inc = recdata[4];	x_inc = recdata[5];
-	if (x_inc <= 0.0 || y_inc <= 0.0)
+	if (x_inc <= 0.0 || y_inc <= 0.0) {
+		gmt_fclose (GMT, fp);
 		return (GMT_GRDIO_BAD_VAL);
-	nx = (int)GMT_get_n (GMT, x_min, x_max, x_inc, 0);
-	if (nx <= 0)
+	}
+	n_columns = (int)gmt_M_get_n (GMT, x_min, x_max, x_inc, 0);
+	if (n_columns <= 0) {
+		gmt_fclose (GMT, fp);
 		return (GMT_GRDIO_BAD_VAL);
-	ny = (int)GMT_get_n (GMT, y_min, y_max, y_inc, 0);
-	if (ny <= 0)
+	}
+	n_rows = (int)gmt_M_get_n (GMT, y_min, y_max, y_inc, 0);
+	if (n_rows <= 0) {
+		gmt_fclose (GMT, fp);
 		return (GMT_GRDIO_BAD_VAL);
+	}
+	gmt_fclose (GMT, fp);
 	/* OK so far; see if file size matches the predicted size given the header info */
-	predicted_size = lrint (ceil ((double)ny /ZBLOCKHEIGHT) * ceil ((double)nx / ZBLOCKWIDTH)) * (ZBLOCKHEIGHT * ZBLOCKWIDTH + PREHEADSIZE + POSTHEADSIZE) * sizeof (float);
+	predicted_size = lrint (ceil ((double)n_rows /ZBLOCKHEIGHT) * ceil ((double)n_columns / ZBLOCKWIDTH)) * (ZBLOCKHEIGHT * ZBLOCKWIDTH + PREHEADSIZE + POSTHEADSIZE) * sizeof (float);
 	if (predicted_size == buf.st_size) {
 		/* Yes, appears to be an AGC grid */
 		header->type = GMT_GRID_IS_AF;
@@ -160,22 +173,25 @@ int GMT_is_agc_grid (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
 	return GMT_GRDIO_BAD_VAL;
 }
 
-int GMT_agc_read_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
-{	/* Read header info. NOTE: All AGC files are assumed to be gridline-registered */
+int gmt_agc_read_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
+	/* Read header info. NOTE: All AGC files are assumed to be gridline-registered */
 	unsigned int i;
 	FILE *fp = NULL;
 	float recdata[RECORDLENGTH], agchead[BUFFHEADSIZE];
 
 	if (!strcmp (header->name, "=")) {
 #ifdef SET_IO_MODE
-		GMT_setmode (GMT, GMT_IN);
+		gmt_setmode (GMT, GMT_IN);
 #endif
 		fp = GMT->session.std[GMT_IN];
 	}
-	else if ((fp = GMT_fopen (GMT, header->name, "rb")) == NULL)
+	else if ((fp = gmt_fopen (GMT, header->name, "rb")) == NULL)
 		return (GMT_GRDIO_OPEN_FAILED);
 
-	if (GMT_fread (recdata, sizeof(float), RECORDLENGTH, fp) < RECORDLENGTH) return (GMT_GRDIO_READ_FAILED);
+	if (gmt_M_fread (recdata, sizeof(float), RECORDLENGTH, fp) < RECORDLENGTH) {
+		gmt_fclose (GMT, fp);
+		return (GMT_GRDIO_READ_FAILED);
+	}
 	
 	header->registration = GMT_GRID_NODE_REG;	/* Hardwired since no info about this in the header */
 	header->wesn[XLO]  = recdata[2];
@@ -184,45 +200,48 @@ int GMT_agc_read_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
 	header->wesn[YHI]  = recdata[1];
 	header->inc[GMT_Y] = recdata[4];
 	header->inc[GMT_X] = recdata[5];
-	header->nx = GMT_grd_get_nx (GMT, header);
-	header->ny = GMT_grd_get_ny (GMT, header);
+	header->n_columns = gmt_M_grd_get_nx (GMT, header);
+	header->n_rows = gmt_M_grd_get_ny (GMT, header);
 	header->z_scale_factor = 1.0;
 	header->z_add_offset = 0.0;
 	for (i = 6; i < PREHEADSIZE; i++) agchead[i-6] = recdata[i];
 	agchead[BUFFHEADSIZE-2] = recdata[RECORDLENGTH-2];
 	agchead[BUFFHEADSIZE-1] = recdata[RECORDLENGTH-1];
-	SaveAGCHeader (header->remark, agchead);
+	agc_save_header (header->remark, agchead);
 	
-	GMT_fclose (GMT, fp);
+	gmt_fclose (GMT, fp);
 
 	return (GMT_NOERROR);
 }
 
-int GMT_agc_write_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header)
-{	/* Write grd header info to file */
+int gmt_agc_write_grd_info (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header) {
+	/* Write grd header info to file */
 	FILE *fp = NULL;
 	float prez[PREHEADSIZE], postz[POSTHEADSIZE];
 
 	if (!strcmp (header->name, "=")) {
 #ifdef SET_IO_MODE
-		GMT_setmode (GMT, GMT_OUT);
+		gmt_setmode (GMT, GMT_OUT);
 #endif
 		fp = GMT->session.std[GMT_OUT];
 	}
-	else if ((fp = GMT_fopen (GMT, header->name, "rb+")) == NULL && (fp = GMT_fopen (GMT, header->name, "wb")) == NULL)
+	else if ((fp = gmt_fopen (GMT, header->name, "rb+")) == NULL && (fp = gmt_fopen (GMT, header->name, "wb")) == NULL)
 		return (GMT_GRDIO_CREATE_FAILED);
 	
-	packAGCheader (prez, postz, header);	/* Stuff header info into the AGC arrays */
+	agc_pack_header (prez, postz, header);	/* Stuff header info into the AGC arrays */
 
-	if (GMT_fwrite (prez, sizeof(float), PREHEADSIZE, fp) < PREHEADSIZE) return (GMT_GRDIO_WRITE_FAILED);
+	if (gmt_M_fwrite (prez, sizeof(float), PREHEADSIZE, fp) < PREHEADSIZE) {
+		gmt_fclose (GMT, fp);
+		return (GMT_GRDIO_WRITE_FAILED);
+	}
 
-	GMT_fclose (GMT, fp);
+	gmt_fclose (GMT, fp);
 
 	return (GMT_NOERROR);
 }
 
-int GMT_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode)
-{	/* header:     	grid structure header
+int gmt_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
+	/* header:     	grid structure header
 	 * grid:	array with final grid
 	 * wesn:	Sub-region to extract  [Use entire file if 0,0,0,0]
 	 * padding:	# of empty rows/columns to add on w, e, s, n of grid, respectively
@@ -246,15 +265,15 @@ int GMT_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 	
 	if (!strcmp (header->name, "=")) {	/* Read from pipe */
 #ifdef SET_IO_MODE
-		GMT_setmode (GMT, GMT_IN);
+		gmt_setmode (GMT, GMT_IN);
 #endif
 		fp = GMT->session.std[GMT_IN];
 	}
-	else if ((fp = GMT_fopen (GMT, header->name, "rb")) == NULL)
+	else if ((fp = gmt_fopen (GMT, header->name, "rb")) == NULL)
 		return (GMT_GRDIO_OPEN_FAILED);
 
-	GMT_err_pass (GMT, GMT_grd_prep_io (GMT, header, wesn, &width_in, &height_in, &first_col, &last_col, &first_row, &last_row, &k), header->name);
-	(void)GMT_init_complex (header, complex_mode, &imag_offset);	/* Set offset for imaginary complex component */
+	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_in, &height_in, &first_col, &last_col, &first_row, &last_row, &k), header->name);
+	(void)gmtlib_init_complex (header, complex_mode, &imag_offset);	/* Set offset for imaginary complex component */
 
 	width_out = width_in;		/* Width of output array */
 	if (pad[XLO] > 0) width_out += pad[XLO];
@@ -264,28 +283,36 @@ int GMT_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 
 	/* Rows are read south to north */
 	
-	GMT_memset (z, ZBLOCKWIDTH * ZBLOCKHEIGHT, float); /* Initialize buffer to zero */
+	gmt_M_memset (z, ZBLOCKWIDTH * ZBLOCKHEIGHT, float); /* Initialize buffer to zero */
 
 	header->z_min = +DBL_MAX;	header->z_max = -DBL_MAX;
+	header->has_NaNs = GMT_GRID_NO_NANS;	/* We are about to check for NaNs and if none are found we retain 1, else 2 */
 	
-	n_blocks_y = urint (ceil ((double)header->ny / (double)ZBLOCKHEIGHT));
-	n_blocks_x = urint (ceil ((double)header->nx / (double)ZBLOCKWIDTH));
+	n_blocks_y = urint (ceil ((double)header->n_rows / (double)ZBLOCKHEIGHT));
+	n_blocks_x = urint (ceil ((double)header->n_columns / (double)ZBLOCKWIDTH));
 	n_blocks = n_blocks_x * n_blocks_y;
 	datablockcol = datablockrow = 0;
 	for (block = 0; block < n_blocks; block++) {
-		if (ReadRecord (fp, z)) return (GMT_GRDIO_READ_FAILED);
+		if (agc_read_record (fp, z)) {
+			gmt_M_free (GMT, k);
+			gmt_fclose (GMT, fp);
+			return (GMT_GRDIO_READ_FAILED);
+		}
 		rowstart = datablockrow * ZBLOCKHEIGHT;
-		rowend = MIN (rowstart + ZBLOCKHEIGHT, header->ny);
+		rowend = MIN (rowstart + ZBLOCKHEIGHT, header->n_rows);
 		for (i = 0, row = rowstart; row < rowend; i++, row++) {
-			j_gmt = header->ny - 1 - row;	/* GMT internal row number */
+			j_gmt = header->n_rows - 1 - row;	/* GMT internal row number */
 			if (j_gmt < first_row || j_gmt > last_row) continue;
 			colstart = datablockcol * ZBLOCKWIDTH;
-			colend = MIN (colstart + ZBLOCKWIDTH, header->nx);
+			colend = MIN (colstart + ZBLOCKWIDTH, header->n_columns);
 			for (j = 0, col = colstart; col < colend; j++, col++) {
 				if (col < first_col || col > last_col) continue;
 				ij = imag_offset + (((j_gmt - first_row) + pad[YHI]) * width_out + col - first_col) + pad[XLO];
 				grid[ij] = (z[j][i] == 0.0) ? GMT->session.f_NaN : z[j][i];	/* AGC uses exact zero as NaN flag */
-				if (GMT_is_fnan (grid[ij])) continue;
+				if (gmt_M_is_fnan (grid[ij])) {
+					header->has_NaNs = GMT_GRID_HAS_NANS;
+					continue;
+				}
 				header->z_min = MIN (header->z_min, (double)grid[ij]);
 				header->z_max = MAX (header->z_max, (double)grid[ij]);
 			}
@@ -296,18 +323,18 @@ int GMT_agc_read_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, floa
 			datablockcol++;
 		}
 	}
-	GMT_free (GMT, k);
+	gmt_M_free (GMT, k);
 
-	header->nx = width_in;	header->ny = height_in;
-	GMT_memcpy (header->wesn, wesn, 4, double);
+	header->n_columns = width_in;	header->n_rows = height_in;
+	gmt_M_memcpy (header->wesn, wesn, 4, double);
 
-	GMT_fclose (GMT, fp);
+	gmt_fclose (GMT, fp);
 	
 	return (GMT_NOERROR);
 }
 
-int GMT_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode)
-{	/* header:	grid structure header
+int gmt_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, float *grid, double wesn[], unsigned int *pad, unsigned int complex_mode) {
+	/* header:	grid structure header
 	 * grid:	array with final grid
 	 * wesn:	Sub-region to write out  [Use entire file if 0,0,0,0]
 	 * padding:	# of empty rows/columns to add on w, e, s, n of grid, respectively
@@ -332,21 +359,21 @@ int GMT_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 
 	if (!strcmp (header->name, "=")) {	/* Write to pipe */
 #ifdef SET_IO_MODE
-		GMT_setmode (GMT, GMT_OUT);
+		gmt_setmode (GMT, GMT_OUT);
 #endif
 		fp = GMT->session.std[GMT_OUT];
 	}
-	else if ((fp = GMT_fopen (GMT, header->name, "wb")) == NULL)
+	else if ((fp = gmt_fopen (GMT, header->name, "wb")) == NULL)
 		return (GMT_GRDIO_CREATE_FAILED);
 	
-	GMT_err_pass (GMT, GMT_grd_prep_io (GMT, header, wesn, &width_out, &height_out, &first_col, &last_col, &first_row, &last_row, &k), header->name);
-	(void)GMT_init_complex (header, complex_mode, &imag_offset);	/* Set offset for imaginary complex component */
+	gmt_M_err_pass (GMT, gmt_grd_prep_io (GMT, header, wesn, &width_out, &height_out, &first_col, &last_col, &first_row, &last_row, &k), header->name);
+	(void)gmtlib_init_complex (header, complex_mode, &imag_offset);	/* Set offset for imaginary complex component */
 
 	width_in = width_out;		/* Physical width of input array */
 	if (pad[XLO] > 0) width_in += pad[XLO];
 	if (pad[XHI] > 0) width_in += pad[XHI];
 
-	GMT_memcpy (header->wesn, wesn, 4, double);
+	gmt_M_memcpy (header->wesn, wesn, 4, double);
 
 	/* Find z_min/z_max */
 
@@ -355,7 +382,7 @@ int GMT_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 		ij = imag_offset + j2 * width_in;
 		for (i = first_col, i2 = pad[XLO]; i <= last_col; i++, i2++) {
 			kk = ij + i2;
-			if (GMT_is_fnan (grid[kk]))	/* in AGC, NaN <--> 0.0 */
+			if (gmt_M_is_fnan (grid[kk]))	/* in AGC, NaN <--> 0.0 */
 				grid[ij] = 0.0f;
 			else {
 				header->z_min = MIN (header->z_min, (double)grid[kk]);
@@ -366,27 +393,27 @@ int GMT_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 
 	/* Since AGC files are always gridline-registered we must change -R when a pixel grid is to be written */
 	if (header->registration == GMT_GRID_PIXEL_REG) {
-		GMT_change_grdreg (GMT, header, GMT_GRID_NODE_REG);
+		gmt_change_grdreg (GMT, header, GMT_GRID_NODE_REG);
 		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Warning: AGC grids are always gridline-registered.  Your pixel-registered grid will be converted.\n");
 		GMT_Report (GMT->parent, GMT_MSG_VERBOSE, "Warning: AGC grid region in file %s reset to %g/%g/%g/%g\n", header->name, header->wesn[XLO], header->wesn[XHI], header->wesn[YLO], header->wesn[YHI]);
 	}
 
-	packAGCheader (prez, postz, header);	/* Stuff header info into the AGC arrays */
+	agc_pack_header (prez, postz, header);	/* Stuff header info into the AGC arrays */
 
-	GMT_memset (outz, ZBLOCKWIDTH * ZBLOCKHEIGHT, float); /* or WriteRecord (fp, outz, prez, postz); points to uninitialised buffer */
+	gmt_M_memset (outz, ZBLOCKWIDTH * ZBLOCKHEIGHT, float); /* or agc_write_record (fp, outz, prez, postz); points to uninitialised buffer */
 
-	n_blocks_y = urint (ceil ((double)header->ny / (double)ZBLOCKHEIGHT));
-	n_blocks_x = urint (ceil ((double)header->nx / (double)ZBLOCKWIDTH));
+	n_blocks_y = urint (ceil ((double)header->n_rows / (double)ZBLOCKHEIGHT));
+	n_blocks_x = urint (ceil ((double)header->n_columns / (double)ZBLOCKWIDTH));
 	n_blocks = n_blocks_x * n_blocks_y;
 	datablockcol = datablockrow = 0;
 	for (block = 0; block < n_blocks; block++) {
 		rowstart = datablockrow * ZBLOCKHEIGHT;
-		rowend = MIN (rowstart + ZBLOCKHEIGHT, header->ny);
+		rowend = MIN (rowstart + ZBLOCKHEIGHT, header->n_rows);
 		for (i = 0, row = rowstart; row < rowend; i++, row++) {
-			j_gmt = header->ny - 1 - row;	/* GMT internal row number */
+			j_gmt = header->n_rows - 1 - row;	/* GMT internal row number */
 			if (j_gmt < first_row || j_gmt > last_row) continue;
 			colstart = datablockcol * ZBLOCKWIDTH;
-			colend = MIN (colstart + ZBLOCKWIDTH, header->nx);
+			colend = MIN (colstart + ZBLOCKWIDTH, header->n_columns);
 			for (j = 0, col = colstart; col < colend; j++, col++) {
 				if (col < first_col || col > last_col) continue;
 				ij = imag_offset + ((j_gmt - first_row) + pad[YHI]) * width_in + (col - first_col) + pad[XLO];
@@ -394,16 +421,20 @@ int GMT_agc_write_grd (struct GMT_CTRL *GMT, struct GMT_GRID_HEADER *header, flo
 			}
 		}
 
-		if (WriteRecord (fp, outz, prez, postz)) return (GMT_GRDIO_WRITE_FAILED);
+		if (agc_write_record (fp, outz, prez, postz)) {
+			gmt_M_free (GMT, k);
+			gmt_fclose (GMT, fp);
+			return (GMT_GRDIO_WRITE_FAILED);
+		}
 
 		if (++datablockrow >= n_blocks_y) {
 			datablockrow = 0;
 			datablockcol++;
 		}
 	}
-	GMT_free (GMT, k);
+	gmt_M_free (GMT, k);
 
-	GMT_fclose (GMT, fp);
+	gmt_fclose (GMT, fp);
 
 	return (GMT_NOERROR);
 }
