@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_error_codes.c 12822 2014-01-31 23:39:56Z remko $
+ *	$Id: gmt_error_codes.c 17449 2017-01-16 21:27:04Z pwessel $
  *
- *	Copyright (c) 1991-2014 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
  * Version:	5 API
  */
 
-const char *g_api_error_string[] = {
+const char *gmt_api_error_string[] = {
 	/* GMT_OK */				"GMT_OK",
 	/* GMT_WRONG_MATRIX_SHAPE */		"GMT_WRONG_MATRIX_SHAPE",
 	/* GMT_ACCESS_NOT_ENABLED */		"GMT_ACCESS_NOT_ENABLED",
@@ -48,6 +48,7 @@ const char *g_api_error_string[] = {
 	/* GMT_ID_TOO_LARGE */			"GMT_ID_TOO_LARGE",
 	/* GMT_IMAGE_BC_ERROR */		"GMT_IMAGE_BC_ERROR",
 	/* GMT_IMAGE_READ_ERROR */		"GMT_IMAGE_READ_ERROR",
+	/* GMT_IMAGE_WRITE_ERROR */		"GMT_IMAGE_WRITE_ERROR",
 	/* GMT_MEMORY_ERROR */			"GMT_MEMORY_ERROR",
 	/* GMT_FREE_EXTERNAL_NOT_ALLOWED */	"GMT_FREE_EXTERNAL_NOT_ALLOWED",
 	/* GMT_FREE_WRONG_LEVEL */		"GMT_FREE_WRONG_LEVEL",
@@ -62,6 +63,7 @@ const char *g_api_error_string[] = {
 	/* GMT_NOT_A_VALID_METHOD */		"GMT_NOT_A_VALID_METHOD",
 	/* GMT_NOT_A_VALID_MODE */		"GMT_NOT_A_VALID_MODE",
 	/* GMT_NOT_A_VALID_MODULE */		"GMT_NOT_A_VALID_MODULE",
+	/* GMT_NOT_A_VALID_PARAMETER */		"GMT_NOT_A_VALID_PARAMETER",
 	/* GMT_NOT_A_VALID_TYPE */		"GMT_NOT_A_VALID_TYPE",
 	/* GMT_NOT_INPUT_OBJECT */		"GMT_NOT_INPUT_OBJECT",
 	/* GMT_NOT_OUTPUT_OBJECT */		"GMT_NOT_OUTPUT_OBJECT",
@@ -70,6 +72,7 @@ const char *g_api_error_string[] = {
 	/* GMT_NO_OUTPUT */			"GMT_NO_OUTPUT",
 	/* GMT_NO_PARAMETERS */			"GMT_NO_PARAMETERS",
 	/* GMT_NO_RESOURCES */			"GMT_NO_RESOURCES",
+	/* GMT_NO_CPT */			"GMT_NO_CPT",
 	/* GMT_N_COLS_NOT_SET */		"GMT_N_COLS_NOT_SET",
 	/* GMT_N_COLS_VARY */			"GMT_N_COLS_VARY",
 	/* GMT_N_ROWS_NOT_SET */		"GMT_N_ROWS_NOT_SET",
@@ -86,6 +89,7 @@ const char *g_api_error_string[] = {
 	/* GMT_PADDING_NOT_ALLOWED */		"GMT_PADDING_NOT_ALLOWED",
 	/* GMT_PARSE_ERROR */			"GMT_PARSE_ERROR",
 	/* GMT_PROG_NOT_FOUND */		"GMT_PROG_NOT_FOUND",
+	/* GMT_PROJECTION_ERROR */		"GMT_PROJECTION_ERROR",
 	/* GMT_PTR_IS_NULL */			"GMT_PTR_IS_NULL",
 	/* GMT_PTR_NOT_NULL */			"GMT_PTR_NOT_NULL",
 	/* GMT_PTR_NOT_UNIQUE */		"GMT_PTR_NOT_UNIQUE",
