@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmt_io.h 12095 2013-09-01 21:14:06Z pwessel $
+ *	$Id: gmt_io.h 17449 2017-01-16 21:27:04Z pwessel $
  *
- *	Copyright (c) 1991-2013 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -16,11 +16,11 @@
  *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
- * Table input/output in GMT can be either ascii or binary (where supported)
+ * Table input/output in GMT can be either ASCII or binary (where supported)
  * and ASCII tables may consist of single or multiple segments.  When the
  * latter is the case usually there is a -M option to signal this case.
  * The structure GMT_IO holds parameters that are used during the reading
- * and processing of ascii tables.
+ * and processing of ASCII tables.
  *
  * Author:	Paul Wessel
  * Date:	15-NOV-2009
@@ -28,25 +28,45 @@
  *
  */
 
+/*!
+ * \file gmt_io.h
+ * \brief  
+ */
+
 #ifndef _GMT_IO_H
 #define _GMT_IO_H
 
+#ifdef HAVE_SETLOCALE
+#	include <locale.h>
+#endif
+
+static inline const char* __gmt_token_separators (void) {
+	static const char separators[] = ",; \t";
+#ifdef HAVE_SETLOCALE
+	struct lconv *lc = localeconv();
+	if ( (strcmp (lc->decimal_point, ",") == 0) )
+		return separators + 1; /* Omit comma */
+#endif
+	return separators;
+}
+#define GMT_TOKEN_SEPARATORS __gmt_token_separators() /* Data columns may be separated by any of these characters */
+
 /* Must add M, m, E, Z, and/or S to the common option processing list */
 #define GMT_OPT(opt) opt
 
-/* Three different i/o status: unused, actively using, or used */
+/*! Three different i/o status: unused, actively using, or used */
 enum GMT_enum_status {
 	GMT_IS_UNUSED = 0,	/* We have not yet read from/written to this resource */
 	GMT_IS_USING,		/* Means we have started reading from/writing to this file */
 	GMT_IS_USED};		/* Means we are done reading from/writing to this file */
 
-/* THere are three GMT/OGR status values */
+/*! There are three GMT/OGR status values */
 enum GMT_ogr_status {
 	GMT_OGR_UNKNOWN = -1,	/* We have not parsed enough records to know yet */
 	GMT_OGR_FALSE,		/* This is NOT a GMT/OGR file */
 	GMT_OGR_TRUE};		/* This is a GMT/OGR file */
 
-#define GMT_polygon_is_hole(S) (S->pol_mode == GMT_IS_HOLE || (S->ogr && S->ogr->pol_mode == GMT_IS_HOLE))
+#define gmt_M_polygon_is_hole(S) (S->pol_mode == GMT_IS_HOLE || (S->ogr && S->ogr->pol_mode == GMT_IS_HOLE))
 
 /* Specific feature geometries as obtained from OGR */
 /* Note: As far as registering or reading data, GMT only needs to know if data type is POINT, LINE, or POLY */
@@ -58,68 +78,62 @@ enum GMT_enum_ogr {
 	GMT_IS_MULTILINESTRING,
 	GMT_IS_MULTIPOLYGON};
 
-/* Codes for aspatial assocation with segment header options: */
-
+/*! Codes for aspatial association with segment header options: */
 enum GMT_enum_segopt {
-	GMT_IS_D = -1,	/* -D */
-	GMT_IS_G = -2,			/* -G */
-	GMT_IS_I = -3,			/* -I */
-	GMT_IS_L = -4,			/* -L */
-	GMT_IS_T = -5,			/* -T */
-	GMT_IS_W = -6,			/* -W */
-	GMT_IS_Z = -7};			/* -Z */
+	/*! -D */	GMT_IS_D = -1,
+	/*! -G */	GMT_IS_G = -2,
+	/*! -I */	GMT_IS_I = -3,
+	/*! -L */	GMT_IS_L = -4,
+	/*! -T */	GMT_IS_T = -5,
+	/*! -W */	GMT_IS_W = -6,
+	/*! -Z */	GMT_IS_Z = -7};
 
 /* Macros to simplify check for return status */
-#define GMT_REC_IS_TABLE_HEADER(C)	(C->current.io.status & GMT_IO_TABLE_HEADER)
-#define GMT_REC_IS_SEGMENT_HEADER(C)	(C->current.io.status & GMT_IO_SEGMENT_HEADER)
-#define GMT_REC_IS_ANY_HEADER(C)	(C->current.io.status & GMT_IO_ANY_HEADER)
-#define GMT_REC_IS_ERROR(C)		(C->current.io.status & GMT_IO_MISMATCH)
-#define GMT_REC_IS_EOF(C)		(C->current.io.status & GMT_IO_EOF)
-#define GMT_REC_IS_NAN(C)		(C->current.io.status & GMT_IO_NAN)
-#define GMT_REC_IS_GAP(C)		(C->current.io.status & GMT_IO_GAP)
-#define GMT_REC_IS_NEW_SEGMENT(C)	(C->current.io.status & GMT_IO_NEW_SEGMENT)
-#define GMT_REC_IS_LINE_BREAK(C)	(C->current.io.status & GMT_IO_LINE_BREAK)
-#define GMT_REC_IS_FILE_BREAK(C)	(C->current.io.status & GMT_IO_NEXT_FILE)
-#define GMT_REC_IS_DATA(C)		(C->current.io.status == 0 || C->current.io.status == GMT_IO_NAN)
+#define gmt_M_rec_is_table_header(C)	(C->current.io.status & GMT_IO_TABLE_HEADER)
+#define gmt_M_rec_is_segment_header(C)	(C->current.io.status & GMT_IO_SEGMENT_HEADER)
+#define gmt_M_rec_is_any_header(C)	(C->current.io.status & GMT_IO_ANY_HEADER)
+#define gmt_M_rec_is_error(C)		(C->current.io.status & GMT_IO_MISMATCH)
+#define gmt_M_rec_is_eof(C)		(C->current.io.status & GMT_IO_EOF)
+#define gmt_M_rec_is_nan(C)		(C->current.io.status & GMT_IO_NAN)
+#define gmt_M_rec_is_gap(C)		(C->current.io.status & GMT_IO_GAP)
+#define gmt_M_rec_is_new_segment(C)	(C->current.io.status & GMT_IO_NEW_SEGMENT)
+#define gmt_M_rec_is_line_break(C)	(C->current.io.status & GMT_IO_LINE_BREAK)
+#define gmt_M_rec_is_file_break(C)	(C->current.io.status & GMT_IO_NEXT_FILE)
+#define gmt_M_rec_is_data(C)		(C->current.io.status == 0 || C->current.io.status == GMT_IO_NAN)
 
 /* Get current setting for in/out columns */
 
-/* Determine if current binary table has header */
-#define GMT_binary_header(GMT,dir) (GMT->common.b.active[dir] && GMT->current.setting.io_header[dir] && GMT->current.setting.io_n_header_items)
-
-/* Types of possible column entries in a file: */
-
-enum GMT_col_enum {
-	GMT_IS_NAN   =   0,	/* Returned by GMT_scanf routines when read fails */
-	GMT_IS_FLOAT		=   1,	/* Generic (double) data type, no special format */
+/*! Types of possible column entries in a file: */
+enum gmt_M_col_enum {
+	GMT_IS_NAN   =   0,	/* Returned by gmt_scanf routines when read fails */
+	GMT_IS_FLOAT	=   1,	/* Generic (double) data type, no special format */
 	GMT_IS_LAT		=   2,
 	GMT_IS_LON		=   4,
 	GMT_IS_GEO		=   6,	/* data type is either Lat or Lon */
 	GMT_IS_RELTIME		=   8,	/* For I/O of data in user units */
 	GMT_IS_ABSTIME		=  16,	/* For I/O of data in calendar+clock units */
 	GMT_IS_RATIME		=  24,	/* To see if time is either Relative or Absolute */
-	GMT_IS_ARGTIME		=  32,	/* To invoke GMT_scanf_argtime()  */
+	GMT_IS_ARGTIME		=  32,	/* To invoke gmt_scanf_argtime()  */
 	GMT_IS_DIMENSION	=  64,	/* A float with [optional] unit suffix, e.g., 7.5c, 0.4i; convert to inch  */
-	GMT_IS_GEOANGLE		= 128,	/* An angle to be converted via map projection to angle on map  */
-	GMT_IS_STRING		= 256,	/* An text argument [internally used, not via -f]  */
-	GMT_IS_UNKNOWN		= 512};	/* Input type is not knowable without -f */
-
-/* Various ways to report longitudes */
+	GMT_IS_GEODIMENSION	= 128,	/* A float with [optional] geo-distance unit suffix, e.g., 7.5n, 0.4d; convert to km  */
+	GMT_IS_GEOANGLE		= 256,	/* An angle to be converted via map projection to angle on map  */
+	GMT_IS_STRING		= 512,	/* An text argument [internally used, not via -f]  */
+	GMT_IS_UNKNOWN		= 1024};	/* Input type is not knowable without -f */
 
+/*! Various ways to report longitudes */
 enum GMT_lon_enum {
 	GMT_IS_GIVEN_RANGE 			= 0,	/* Report lon as is */
-	GMT_IS_0_TO_P360_RANGE			= 1,	/* Report 0 <= lon <= 360 */
+	GMT_IS_0_TO_P360_RANGE		= 1,	/* Report 0 <= lon <= 360 */
 	GMT_IS_0_TO_P360			= 2,	/* Report 0 <= lon < 360 */
-	GMT_IS_M360_TO_0_RANGE			= 3,	/* Report -360 <= lon <= 0 */
+	GMT_IS_M360_TO_0_RANGE		= 3,	/* Report -360 <= lon <= 0 */
 	GMT_IS_M360_TO_0			= 4,	/* Report -360 < lon <= 0 */
-	GMT_IS_M180_TO_P180_RANGE		= 5,	/* Report -180 <= lon <= +180 */
+	GMT_IS_M180_TO_P180_RANGE	= 5,	/* Report -180 <= lon <= +180 */
 	GMT_IS_M180_TO_P180			= 6,	/* Report -180 <= lon < +180 */
-	GMT_IS_M180_TO_P270_RANGE		= 7};	/* Report -180 <= lon < +270 [GSHHS only] */
-
-/* How to handle NaNs in records */
+	GMT_IS_M180_TO_P270_RANGE	= 7};	/* Report -180 <= lon < +270 [GSHHG only] */
 
+/*! How to handle NaNs in records */
 enum GMT_io_nan_enum {
-	GMT_IO_NAN_OK = 0,	/* NaNs are fine; just ouput the record as is */
+	GMT_IO_NAN_OK = 0,	/* NaNs are fine; just output the record as is */
 	GMT_IO_NAN_SKIP,	/* -s[cols]	: Skip records with z == NaN in selected cols [z-col only] */
 	GMT_IO_NAN_KEEP,	/* -sr		: Skip records with z != NaN */
 	GMT_IO_NAN_ONE};	/* -sa		: Skip records with at least one NaN */
@@ -147,13 +161,10 @@ enum GMT_io_nan_enum {
 	typedef long off_t;
 #endif /* HAVE__FSEEKI64 && HAVE__FTELLI64 */
 
-#define GMT_fdopen(handle, mode) fdopen(handle, mode)
-#define GMT_fgetc(stream) fgetc(stream)
-#define GMT_ungetc(c, stream) ungetc(c, stream)
-#define GMT_fputs(line,fp) fputs(line,fp)
-#define GMT_fread(ptr,size,nmemb,stream) fread(ptr,size,nmemb,stream)
-#define GMT_fwrite(ptr,size,nmemb,stream) fwrite(ptr,size,nmemb,stream)
-#define GMT_rewind(stream) rewind(stream)
+#define gmt_M_fputs(line,fp) fputs(line,fp)
+#define gmt_M_fread(ptr,size,nmemb,stream) fread(ptr,size,nmemb,stream)
+#define gmt_M_fwrite(ptr,size,nmemb,stream) fwrite(ptr,size,nmemb,stream)
+#define gmt_M_rewind(stream) rewind(stream)
 
 /* Low-level structures used internally */
 
@@ -177,7 +188,7 @@ struct GMT_CLOCK_IO {
 
 struct GMT_DATE_IO {
 	bool skip;			/* Only true if a format string was pass as NULL */
-	unsigned int T_pos;		/* String position of the expected 'T' marker (INPUT only) */
+	bool watch;			/* Only true if input format has month last and is monthname */
 	int item_order[4];		/* The sequence year, month, day, day-of-year in input calendar string (-ve if unused) */
 	int item_pos[4];		/* Which position year, month, day, day-of-year has in calendar string (-ve if unused) */
 	bool Y2K_year;		/* true if we have 2-digit years */
@@ -218,8 +229,8 @@ struct GMT_COL_TYPE {	/* Used by -b for binary formatting */
 };
 
 struct GMT_IO {				/* Used to process input data records */
-	void * (*input) (struct GMT_CTRL *, FILE *, uint64_t *, int *);	/* Pointer to function reading ascii or binary tables */
-	int (*output) (struct GMT_CTRL *, FILE *, uint64_t, double *);	/* Pointer to function writing ascii or binary tables */
+	void * (*input) (struct GMT_CTRL *, FILE *, uint64_t *, int *);	/* Pointer to function reading ASCII or binary tables */
+	int (*output) (struct GMT_CTRL *, FILE *, uint64_t, double *);	/* Pointer to function writing ASCII or binary tables */
 	int (*read_item) (struct GMT_CTRL *, FILE *, uint64_t, double *);		/* Pointer to function reading 1-col z tables in grd2xyz */
 	int (*write_item) (struct GMT_CTRL *, FILE *, uint64_t, double *);		/* Pointer to function writing 1-col z tables in xyz2grd */
 	bool (*ogr_parser) (struct GMT_CTRL *, char *);				/* Set to handle either header or data OGR records */
@@ -230,12 +241,14 @@ struct GMT_IO {				/* Used to process input data records */
 	double prev_rec[GMT_MAX_COLUMNS];	/* The previous data record */
 	struct GMT_GRID_INFO grd_info;
 
-	bool multi_segments[2];	/* true if current Ascii input/output file has multiple segments */
+	bool multi_segments[2];	/* true if current ASCII input/output file has multiple segments */
+	bool skip_headers_on_outout;	/* true when gmtconvert -T is set [or possibly other similar actions in the future] */
 	bool skip_bad_records;	/* true if records where x and/or y are NaN or Inf */
 	bool give_report;		/* true if functions should report how many bad records were skipped */
 	bool skip_duplicates;	/* true if we should ignore duplicate x,y records */
-	bool read_mixed;		/* true if we are reading ascii x y [z] [variable numbers of text] */
+	bool read_mixed;		/* true if we are reading ASCII x y [z] [variable numbers of text] */
 	bool need_previous;		/* true if when parsing a record we need access to previous record values (e.g., for gap or duplicate checking) */
+	bool warn_geo_as_cartesion;	/* true if we should warn if we read a record with geographic data while the expected format has not been set (i.e., no -J or -fg) */
 
 	uint64_t seg_no;		/* Number of current multi-segment in entire data set */
 	uint64_t seg_in_tbl_no;		/* Number of current multi-segment in current table */
@@ -252,14 +265,14 @@ struct GMT_IO {				/* Used to process input data records */
 	uint64_t rec_no;		/* Number of current records (counts headers etc) in entire data set */
 	uint64_t rec_in_tbl_no;		/* Number of current record (counts headers etc) in current table */
 	uint64_t pt_no;			/* Number of current valid points in a row  */
-	uint64_t curr_pos[2][3];	/* Keep track of current input/output table, segment, and row (for rec-by-rec action) */
+	int64_t curr_pos[2][4];		/* Keep track of current input/output table, segment, row, and table headers (for rec-by-rec action) */
 	char r_mode[4];			/* Current file opening mode for reading (r or rb) */
 	char w_mode[4];			/* Current file opening mode for writing (w or wb) */
 	char a_mode[4];			/* Current file append mode for writing (a+ or ab+) */
-	char current_record[GMT_BUFSIZ];	/* Current ascii record */
-	char segment_header[GMT_BUFSIZ];	/* Current ascii segment header */
-	char current_filename[2][GMT_BUFSIZ];	/* Current filenames (or <stdin>/<stdout>) */
-	char *o_format[GMT_MAX_COLUMNS];	/* Custom output ascii format to overrule format_float_out */
+	char record[GMT_BUFSIZ];	/* Current ASCII record */
+	char segment_header[GMT_BUFSIZ];	/* Current ASCII segment header */
+	char filename[2][GMT_BUFSIZ];	/* Current filenames (or <stdin>/<stdout>) */
+	char *o_format[GMT_MAX_COLUMNS];	/* Custom output ASCII format to overrule format_float_out */
 	int ncid;			/* NetCDF file ID (when opening netCDF file) */
 	int nvars;			/* Number of requested variablesin netCDF file */
 	uint64_t ncols;			/* Number of total columns in netCDF file */
@@ -295,8 +308,8 @@ struct GMT_Z_IO {		/* Used when processing z(x,y) table input when (x,y) is impl
 	unsigned int x_missing;	/* 1 if a periodic (right) column is implicit (i.e., not stored) */
 	unsigned int y_missing;	/* 1 if a periodic (top) row is implicit (i.e., not stored) */
 	unsigned int format;	/* Either GMT_IS_COL_FORMAT or GMT_IS_ROW_FORMAT */
-	unsigned int x_period;	/* length of a row in the input data ( <= nx, see x_missing) */
-	unsigned int y_period;	/* length of a col in the input data ( <= ny, see y_missing) */
+	unsigned int x_period;	/* length of a row in the input data ( <= n_columns, see x_missing) */
+	unsigned int y_period;	/* length of a col in the input data ( <= n_rows, see y_missing) */
 	unsigned int start_col;	/* First logical column in file */
 	unsigned int start_row;	/* First logical row in file */
 	unsigned int gmt_i;		/* Current column number in the GMT registered grid */
@@ -322,13 +335,6 @@ struct GMT_PLOT_CALCLOCK {
 	struct GMT_GEO_IO geo;
 };
 
-/* Byteswap widths used with gmt_byteswap_file */
-typedef enum {
-	Int16len = 2,
-	Int32len = 4,
-	Int64len = 8
-} SwapWidth;
-
 /* For the GMT_GRID container, see gmt_grdio.h */
 
 #endif /* _GMT_IO_H */
