@@ -1,15 +1,15 @@
-/*	$Id: gshhg.c 14247 2015-04-28 18:46:55Z pwessel $
+/*	$Id: gshhg.c 17543 2017-02-09 14:14:29Z jluis $
  *
- *	Copyright (c) 1996-2015 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1996-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  * PROGRAM:	gshhg.c
  * AUTHOR:	Paul Wessel (pwessel@hawaii.edu)
  * CREATED:	JAN. 28, 1996
- * DATE:	JULY 1, 2014
- * PURPOSE:	To extract ASCII data from the binary GSHHG shoreline data
+ * DATE:	JAN 1, 2015
+ * PURPOSE:	To extract ASCII data from the binary GSHHG geography database
  *		as described in the 1996 Wessel & Smith JGR Data Analysis Note.
- * VERSION:	1-JUL-2014.  For use with GSHHG version 2.3.1
+ * VERSION:	1-JAN-2015.  For use with GSHHG version 2.3.4
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU Lesser General Public License as published by
@@ -25,11 +25,12 @@
 #define THIS_MODULE_NAME	"gshhg"
 #define THIS_MODULE_LIB		"gshhg"
 #define THIS_MODULE_PURPOSE	"Extract data tables from binary GSHHS or WDBII data files"
+#define THIS_MODULE_KEYS	">D},>TL"
 
 #include "gmt_dev.h"
 #include "gmt_gshhg.h"
 
-#define GMT_PROG_OPTIONS "-:Vbo"
+#define GMT_PROG_OPTIONS "-:Vbdo"
 
 struct GSHHG_CTRL {
 	struct In {	/* <file> */
@@ -65,47 +66,45 @@ struct GSHHG_CTRL {
 	} Q;
 };
 
-void *New_gshhg_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
+GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
 	struct GSHHG_CTRL *C;
 
-	C = GMT_memory (GMT, NULL, 1, struct GSHHG_CTRL);
+	C = gmt_M_memory (GMT, NULL, 1, struct GSHHG_CTRL);
 
 	return (C);
 }
 
-void Free_gshhg_Ctrl (struct GMT_CTRL *GMT, struct GSHHG_CTRL *C) {	/* Deallocate control structure */
+GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GSHHG_CTRL *C) {	/* Deallocate control structure */
 	if (!C) return;
-	if (C->In.file) free (C->In.file);	
-	if (C->Out.file) free (C->Out.file);	
-	GMT_free (GMT, C);	
+	gmt_M_str_free (C->In.file);	
+	gmt_M_str_free (C->Out.file);	
+	gmt_M_free (GMT, C);	
 }
 
-int GMT_gshhg_usage (struct GMTAPI_CTRL *API, int level)
-{
-	GMT_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
+	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: gshhg gshhs|wdb_rivers|wdb_borders_[f|h|i|l|c].b [-A<area>] [-G] [-I<id>] [-L] [-N<level>]\n\t[-Qe|i] [%s] [%s] [%s] > table\n", GMT_V_OPT, GMT_bo_OPT, GMT_o_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: gshhg gshhs|wdb_rivers|wdb_borders_[f|h|i|l|c].b [-A<area>] [-G] [-I<id>] [-L] [-N<level>]\n\t[-Qe|i] [%s] [%s] [%s] [%s]\n\n", GMT_V_OPT, GMT_bo_OPT, GMT_do_OPT, GMT_o_OPT);
 
-	if (level == GMT_SYNOPSIS) return (EXIT_FAILURE);
+	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
 	GMT_Message (API, GMT_TIME_NONE, "\tgshhs|wdb_rivers|wdb_borders_[f|h|i|l|c].b is a GSHHG polygon or line file.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\n\tOPTIONS:\n");
         GMT_Message (API, GMT_TIME_NONE, "\t-A Extract polygons whose area is greater than or equal to <area> (in km^2) [all].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-G Write '%%' at start of each segment header [P or L] (overwrites -M)\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t   and write 'NaN NaN' after each segment to enable import by GNU Octave or Matlab.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   and write 'NaN NaN' after each segment to enable import by MATLAB or GNU Octave.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-L List header records only (no data records will be written).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-I Output data for polygon number <id> only.  Use -Ic to get all continent polygons\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   [Default is all polygons].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-N Output features whose level matches <level> [Default outputs all levels].\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t-Q Control river-lakes: Use -Qe to exclude river-lakes, and -Qi to ONLY get river-lakes\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   [Default outputs all polygons].\n");
-	GMT_Option (API, "V,bo2,o,:,.");
+	GMT_Option (API, "V,bo2,do,o,:,.");
 	
-	return (EXIT_FAILURE);
+	return (GMT_MODULE_USAGE);
 }
 	
-int GMT_gshhg_parse (struct GMT_CTRL *GMT, struct GSHHG_CTRL *Ctrl, struct GMT_OPTION *options)
-{
+GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GSHHG_CTRL *Ctrl, struct GMT_OPTION *options) {
 	/* This parses the options provided to gshhg and sets parameters in CTRL.
 	 * Any GMT common options will override values set previously by other commands.
 	 * It also replaces any file names specified as input or output with the data ID
@@ -121,7 +120,7 @@ int GMT_gshhg_parse (struct GMT_CTRL *GMT, struct GSHHG_CTRL *Ctrl, struct GMT_O
 
 			case '<':	/* Count input files */
 				n_files++;
-				if (n_files == 1 && GMT_check_filearg (GMT, '<', opt->arg, GMT_IN, GMT_IS_DATASET))
+				if (n_files == 1 && gmt_check_filearg (GMT, '<', opt->arg, GMT_IN, GMT_IS_DATASET))
 					Ctrl->In.file = strdup (opt->arg);
 				else
 					n_errors++;
@@ -149,14 +148,14 @@ int GMT_gshhg_parse (struct GMT_CTRL *GMT, struct GSHHG_CTRL *Ctrl, struct GMT_O
 					Ctrl->I.mode = 1;
 				else {
 					sval = atoi (opt->arg);
-					n_errors += GMT_check_condition (GMT, sval < 0, "Syntax error -I: ID cannot be negative!\n");
+					n_errors += gmt_M_check_condition (GMT, sval < 0, "Syntax error -I: ID cannot be negative!\n");
 					Ctrl->I.id = sval;
 				}
 				break;
 			case 'N':
 				Ctrl->N.active = true;
 				sval = atoi (opt->arg);
-				n_errors += GMT_check_condition (GMT, sval < 0, "Syntax error -N: Level cannot be negative!\n");
+				n_errors += gmt_M_check_condition (GMT, sval < 0, "Syntax error -N: Level cannot be negative!\n");
 				Ctrl->N.level = sval;
 				break;
 			case 'Q':
@@ -169,23 +168,22 @@ int GMT_gshhg_parse (struct GMT_CTRL *GMT, struct GSHHG_CTRL *Ctrl, struct GMT_O
 					Ctrl->Q.mode = 3;	/* Flag the error */
 				break;
 			default:	/* Report bad options */
-				n_errors += GMT_default_error (GMT, opt->option);
+				n_errors += gmt_default_error (GMT, opt->option);
 				break;
 		}
 	}
 
-	n_errors += GMT_check_condition (GMT, n_files != 1, "Syntax error: No data file specified!\n");
-	n_errors += GMT_check_condition (GMT, Ctrl->A.active && Ctrl->A.min < 0.0, "Syntax error -A: area cannot be negative!\n");
-	n_errors += GMT_check_condition (GMT, Ctrl->Q.active && Ctrl->Q.mode == 3, "Syntax error -Q: Append e or i!\n");
+	n_errors += gmt_M_check_condition (GMT, n_files != 1, "Syntax error: No data file specified!\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->A.active && Ctrl->A.min < 0.0, "Syntax error -A: area cannot be negative!\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->Q.active && Ctrl->Q.mode == 3, "Syntax error -Q: Append e or i!\n");
 
-	return (n_errors ? GMT_PARSE_ERROR : GMT_OK);
+	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
 
-#define bailout(code) {GMT_Free_Options (mode); return (code);}
-#define Return(code) {Free_gshhg_Ctrl (GMT, Ctrl); GMT_end_module (GMT, GMT_cpy); bailout (code);}
+#define bailout(code) {gmt_M_free_options (mode); return (code);}
+#define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
-int GMT_gshhg (void *V_API, int mode, void *args)
-{
+int GMT_gshhg (void *V_API, int mode, void *args) {
 	unsigned int row, seg_no = 0, is_line = 0, n_seg = 0, m, level, this_id;
 	int error, gmode, version, greenwich, is_river, src;
 	int32_t max_east = 270000000;
@@ -217,37 +215,37 @@ int GMT_gshhg (void *V_API, int mode, void *args)
 	struct GSHHG_CTRL *Ctrl = NULL;
 	struct GMT_CTRL *GMT = NULL, *GMT_cpy = NULL;
 	struct GMT_OPTION *options = NULL;
-	struct GMTAPI_CTRL *API = GMT_get_API_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
+	struct GMTAPI_CTRL *API = gmt_get_api_ptr (V_API);	/* Cast from void to GMTAPI_CTRL pointer */
 
 	/*----------------------- Standard module initialization and parsing ----------------------*/
 
 	if (API == NULL) return (GMT_NOT_A_SESSION);
-	if (mode == GMT_MODULE_PURPOSE) return (GMT_gshhg_usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
+	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (GMT_gshhg_usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (GMT_gshhg_usage (API, GMT_SYNOPSIS));		/* Return the synopsis */
+	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
+	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));		/* Return the synopsis */
 
 	/* Parse the command-line arguments */
-	GMT = GMT_begin_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, &GMT_cpy); /* Save current state */
+	GMT = gmt_begin_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, &GMT_cpy); /* Save current state */
 	if (GMT_Parse_Common (API, GMT_PROG_OPTIONS, options)) Return (API->error);
-	Ctrl = New_gshhg_Ctrl (GMT);	/* Allocate and initialize a new control structure */
-	if ((error = GMT_gshhg_parse (GMT, Ctrl, options))) Return (error);
+	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
+	if ((error = parse (GMT, Ctrl, options)) != 0) Return (error);
 	
 	/*---------------------------- This is the gshhg main code ----------------------------*/
 
-	if (GMT_access (GMT, Ctrl->In.file, F_OK)) {
+	if (gmt_access (GMT, Ctrl->In.file, F_OK)) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Cannot find file %s\n", Ctrl->In.file);
-		Return (EXIT_FAILURE);
+		Return (GMT_FILE_NOT_FOUND);
 	}
-	if ((fp = GMT_fopen (GMT, Ctrl->In.file, "rb")) == NULL ) {
+	if ((fp = gmt_fopen (GMT, Ctrl->In.file, "rb")) == NULL ) {
 		GMT_Report (API, GMT_MSG_NORMAL, "Cannot read file %s\n", Ctrl->In.file);
-		Return (EXIT_FAILURE);
+		Return (GMT_ERROR_ON_FOPEN);
 	}
 
-	GMT_set_segmentheader (GMT, GMT_OUT, true);	/* Turn on segment headers on output */
-	GMT_set_geographic (GMT, GMT_IN);
-	GMT_set_geographic (GMT, GMT_OUT);
+	gmt_set_segmentheader (GMT, GMT_OUT, true);	/* Turn on segment headers on output */
+	gmt_set_geographic (GMT, GMT_IN);
+	gmt_set_geographic (GMT, GMT_OUT);
 	if (Ctrl->G.active) {
 		marker = GMT->current.setting.io_seg_marker[GMT_OUT];
 		GMT->current.setting.io_seg_marker[GMT_OUT] = '%';
@@ -259,6 +257,7 @@ int GMT_gshhg (void *V_API, int mode, void *args)
 		dim[GMT_ROW] = n_alloc = (Ctrl->I.active) ? ((Ctrl->I.mode) ? 6 : 1) : GSHHG_MAXPOL;
 		if ((X = GMT_Create_Data (API, GMT_IS_TEXTSET, GMT_IS_NONE, 0, dim, NULL, NULL, 0, 0, NULL)) == NULL) {
 			GMT_Report (API, GMT_MSG_NORMAL, "Unable to create a text set for GSHHG header features.\n");
+			gmt_fclose (GMT, fp);
 			return (API->error);
 		}
 	}
@@ -266,22 +265,23 @@ int GMT_gshhg (void *V_API, int mode, void *args)
 		dim[GMT_SEG] = n_alloc = 0;
 		if ((D = GMT_Create_Data (API, GMT_IS_DATASET, GMT_IS_POLY, 0, dim, NULL, NULL, 0, 0, NULL)) == NULL) {
 			GMT_Report (API, GMT_MSG_NORMAL, "Unable to create a data set for GSHHG features.\n");
+			gmt_fclose (GMT, fp);
 			return (API->error);
 		}
 	}
 	sprintf (header, "# Data extracted from GSHHG file %s", Ctrl->In.file);
 	if (Ctrl->L.active) {	/* Want a text set of headers back */
-		X->table[0]->header = GMT_memory (GMT, NULL, 1, char *);
+		X->table[0]->header = gmt_M_memory (GMT, NULL, 1, char *);
 		X->table[0]->header[0] = strdup (header);
 		X->table[0]->n_headers = 1;
 		TX = X->table[0]->segment[0];	/* There is only one output table with one segment */
 	}
 	else {
-		D->table[0]->header = GMT_memory (GMT, NULL, 1, char *);
+		D->table[0]->header = gmt_M_memory (GMT, NULL, 1, char *);
 		D->table[0]->header[0] = strdup (header);
 		D->table[0]->n_headers = 1;
 		n_alloc = (Ctrl->I.active) ? ((Ctrl->I.mode) ? 6 : 1) : GSHHG_MAXPOL;
-		D->table[0]->segment = GMT_memory (GMT, NULL, n_alloc, struct GMT_DATASEGMENT *);
+		D->table[0]->segment = gmt_M_memory (GMT, NULL, n_alloc, struct GMT_DATASEGMENT *);
 		T = D->table[0]->segment;	/* There is only one output table with one or many segments */
 	}
 	n_read = fread (&h, sizeof (struct GSHHG_HEADER), 1U, fp);
@@ -308,7 +308,6 @@ int GMT_gshhg (void *V_API, int mode, void *args)
 		source = (level > 4) ? 'A' : ((src == 1) ? 'W' : 'C');		/* Either Antarctica, WVS or CIA (WDBII) pedigree */
 		if (is_river) source = (char)tolower ((int)source);	/* Lower case c means river-lake */
 		is_line = (h.area) ? 0 : 1;			/* Either Polygon (0) or Line (1) (if no area) */
-		if (is_line && D->geometry == GMT_IS_POLY) D->geometry = GMT_IS_LINE;	/* Change from polygon to line geometry */
 		if (version >= 9) {				/* Magnitude for area scale */
 			m = h.flag >> 26;
 			scale = pow (10.0, (double)m);		/* Area scale */
@@ -330,24 +329,25 @@ int GMT_gshhg (void *V_API, int mode, void *args)
 		if (Ctrl->L.active) {	/* Want a text set of headers back */
 			if (seg_no == n_alloc) {	/* Must add more segments to this table first */
 				n_alloc <<= 2;
-				TX->record = GMT_memory (GMT, TX->record, n_alloc, char *);
+				TX->data = gmt_M_memory (GMT, TX->data, n_alloc, char *);
 			}
 		}
 		else {
+			if (is_line && D->geometry == GMT_IS_POLY) D->geometry = GMT_IS_LINE;	/* Change from polygon to line geometry */
 			dim[GMT_ROW] = h.n + Ctrl->G.active;	/* Number of data records to allocate for this segment/polygon*/
 			if (seg_no == n_alloc) {	/* Must add more segments to this table first */
 				size_t old_n_alloc = n_alloc;
 				n_alloc <<= 1;
-				T = GMT_memory (GMT, T, n_alloc, struct GMT_DATASEGMENT *);
-				GMT_memset (&(T[old_n_alloc]), n_alloc - old_n_alloc, struct GMT_DATASEGMENT *);	/* Set to NULL */
+				T = gmt_M_memory (GMT, T, n_alloc, struct GMT_DATASEGMENT *);
+				gmt_M_memset (&(T[old_n_alloc]), n_alloc - old_n_alloc, struct GMT_DATASEGMENT *);	/* Set to NULL */
 			}
 		}
 
 		/* Format w/e/s/n for header according to users format choice */
-		GMT_ascii_format_col (GMT, west,  w, GMT_OUT, GMT_X);
-		GMT_ascii_format_col (GMT, east,  e, GMT_OUT, GMT_X);
-		GMT_ascii_format_col (GMT, south, s, GMT_OUT, GMT_Y);
-		GMT_ascii_format_col (GMT, north, n, GMT_OUT, GMT_Y);
+		gmt_ascii_format_col (GMT, west,  w, GMT_OUT, GMT_X);
+		gmt_ascii_format_col (GMT, east,  e, GMT_OUT, GMT_X);
+		gmt_ascii_format_col (GMT, south, s, GMT_OUT, GMT_Y);
+		gmt_ascii_format_col (GMT, north, n, GMT_OUT, GMT_Y);
 		
 		/* Create the segment/polygon header record */
 		if (is_line) {	/* River or border line-segment */
@@ -358,58 +358,55 @@ int GMT_gshhg (void *V_API, int mode, void *args)
 			sprintf (header, "%6d%8d%2d%2c %.12g %.12g %s %s %s %s %6d %6d", h.id, h.n, level, source, area, f_area, west, east, south, north, h.container, h.ancestor);
 
 		if (Ctrl->L.active) {	/* Skip data, only wanted the headers */
-			TX->record[seg_no] = strdup (header);
+			TX->data[seg_no] = strdup (header);
 			TX->n_rows++;
 			fseek (fp, (off_t)(h.n * sizeof(struct GSHHG_POINT)), SEEK_CUR);
 		}
 		else {	/* Return the data points also */
 			/* Place the header in the output data structure */
-			T[seg_no] = GMT_memory (GMT, NULL, 1, struct GMT_DATASEGMENT);
-			T[seg_no]->header = strdup (header);
+			T[seg_no] = GMT_Alloc_Segment (GMT->parent, GMT_IS_DATASET, dim[GMT_ROW], dim[GMT_COL], header, NULL);
 			if (h.id == 0)	/* Special longitude range for Eurasia since it crosses Greenwich and Dateline */
 				T[seg_no]->range = GMT_IS_M180_TO_P270_RANGE;
 			else if (h.id == 4)	/* Special longitude range for Antarctica since it crosses Greenwich and Dateline */
 				T[seg_no]->range = GMT_IS_M180_TO_P180_RANGE;
 			else
 				T[seg_no]->range = (greenwich & 2) ? GMT_IS_0_TO_P360_RANGE : GMT_IS_M180_TO_P180_RANGE;
-			/* Allocate h.n number of data records */
-			GMT_alloc_segment (GMT, T[seg_no], dim[GMT_ROW], dim[GMT_COL], true);
 			for (row = 0; row < h.n; row++) {
 				if (fread (&p, sizeof (struct GSHHG_POINT), 1U, fp) != 1) {
 					GMT_Report (API, GMT_MSG_NORMAL, "Error reading file %s for %s %d, point %d.\n", Ctrl->In.file, name[is_line], h.id, row);
-					Return (EXIT_FAILURE);
+					Return (GMT_DATA_READ_ERROR);
 				}
 				if (must_swab) /* Must deal with different endianness */
 					bswap_POINT_struct (&p);
-				T[seg_no]->coord[GMT_X][row] = p.x * GSHHG_SCL;
-				if ((greenwich && p.x > max_east) || (h.west > 180000000)) T[seg_no]->coord[GMT_X][row] -= 360.0;
-				T[seg_no]->coord[GMT_Y][row] = p.y * GSHHG_SCL;
+				T[seg_no]->data[GMT_X][row] = p.x * GSHHG_SCL;
+				if ((greenwich && p.x > max_east) || (h.west > 180000000)) T[seg_no]->data[GMT_X][row] -= 360.0;
+				T[seg_no]->data[GMT_Y][row] = p.y * GSHHG_SCL;
 			}
-			if (Ctrl->G.active) T[seg_no]->coord[GMT_X][row] = T[seg_no]->coord[GMT_Y][row] = GMT->session.d_NaN;
+			if (Ctrl->G.active) T[seg_no]->data[GMT_X][row] = T[seg_no]->data[GMT_Y][row] = GMT->session.d_NaN;
 			D->n_records += T[seg_no]->n_rows;
 		}
 		seg_no++;
 		max_east = 180000000;	/* Only Eurasia (the first polygon) needs 270 */
 		n_read = fread (&h, sizeof (struct GSHHG_HEADER), 1U, fp);	/* Get the next GSHHG header */
 	}
-	GMT_fclose (GMT, fp);
+	gmt_fclose (GMT, fp);
 	
 	if (Ctrl->L.active) {	/* Skip data, only wanted the headers */
 		if (seg_no < n_alloc) {	/* Allocate to final size table */
-			TX->record = GMT_memory (GMT, TX->record, seg_no, char *);
+			TX->data = gmt_M_memory (GMT, TX->data, seg_no, char *);
 		}
 		X->n_records = X->table[0]->n_records = TX->n_rows;
-		if (GMT_Write_Data (API, GMT_IS_TEXTSET, GMT_IS_FILE, GMT_IS_NONE, GMT_WRITE_SET, NULL, Ctrl->Out.file, X) != GMT_OK) {
+		if (GMT_Write_Data (API, GMT_IS_TEXTSET, GMT_IS_FILE, GMT_IS_NONE, GMT_WRITE_SET, NULL, Ctrl->Out.file, X) != GMT_NOERROR) {
 			Return (API->error);
 		}
 	}
 	else {
 		if (seg_no < n_alloc) {	/* Allocate to final size table */
-			D->table[0]->segment = GMT_memory (GMT, D->table[0]->segment, seg_no, struct GMT_DATASEGMENT *);
+			D->table[0]->segment = gmt_M_memory (GMT, D->table[0]->segment, seg_no, struct GMT_DATASEGMENT *);
 		}
 		D->n_segments = D->table[0]->n_segments = seg_no;
 		gmode = (is_line) ? GMT_IS_LINE : GMT_IS_POLY;
-		if (GMT_Write_Data (API, GMT_IS_DATASET, GMT_IS_FILE, gmode, GMT_WRITE_SET, NULL, Ctrl->Out.file, D) != GMT_OK) {
+		if (GMT_Write_Data (API, GMT_IS_DATASET, GMT_IS_FILE, gmode, GMT_WRITE_SET, NULL, Ctrl->Out.file, D) != GMT_NOERROR) {
 			Return (API->error);
 		}
 	}
@@ -418,5 +415,5 @@ int GMT_gshhg (void *V_API, int mode, void *args)
 
 	if (Ctrl->G.active) GMT->current.setting.io_seg_marker[GMT_OUT] = marker;
 
-	Return (GMT_OK);
+	Return (GMT_NOERROR);
 }
