@@ -1,7 +1,7 @@
 /*-----------------------------------------------------------------
- *	$Id: x2sys.h 12822 2014-01-31 23:39:56Z remko $
+ *	$Id: x2sys.h 17449 2017-01-16 21:27:04Z pwessel $
  *
- *      Copyright (c) 1999-2014 by P. Wessel
+ *      Copyright (c) 1999-2017 by P. Wessel
  *      See LICENSE.TXT file for copying and redistribution conditions.
  *
  *      This program is free software; you can redistribute it and/or modify
@@ -38,9 +38,9 @@
   -------------------
 
     -Type 1:  columns separated by tabs, spaces, or commas:
-	Each field type must be set to 'a' for ascii string
+	Each field type must be set to 'a' for ASCII string
     -Type 2:  FORTRAN card format, all glued together:
-	Each field type must be set to 'A' for ascii string
+	Each field type must be set to 'A' for ASCII string
 	and the start-stop column must be appended at the end of
 	the info line.
 
@@ -63,6 +63,11 @@
   This information must be conveyed via the *.def files (-D)
 ------------------------------------------------------------------*/
 
+/*!
+ * \file x2sys.h
+ * \brief Declaration for the X2SYS and XOVER structures.
+ */
+
 #include "mgd77/mgd77.h"
 #include "gmt_dev.h"
 
@@ -112,11 +117,11 @@ struct X2SYS_INFO {
 	unsigned int *out_order;	/* Array with input column number as function of output column */
 	bool *use_column;		/* Array of T/F for which columns to use */
 	unsigned int geodetic;		/* How longitudes should be stored: 0: (0-360), 1: (-360,0), 2 (-180/+180) */
-	unsigned int dist_flag;		/* How distances are calulated: (0 = Cartesian, 1 = Flat earth, 2 = great circle, 3 = geodesic) */
+	unsigned int dist_flag;		/* How distances are calculated: (0 = Cartesian, 1 = Flat earth, 2 = great circle, 3 = geodesic) */
 	/* read_file is a pointer to function that reads this file */
 	int (*read_file) (struct GMT_CTRL *, char *, double ***, struct X2SYS_INFO *, struct X2SYS_FILE_INFO *, struct GMT_IO *, uint64_t *);
 	unsigned int file_type;		/* 0 = ASCII, 1 = native binary, 2 = netCDF */
-	bool ascii_out;			/* true if output should be in ascii */
+	bool ascii_out;			/* true if output should be in ASCII */
 	bool multi_segment;		/* true if there are multiple segments in this file */
 	bool geographic;		/* true if x/y data are lon/lat */
 	bool ms_next;			/* true if we just read 1st record in a new segments in this file */
@@ -126,6 +131,7 @@ struct X2SYS_INFO {
 	char suffix[16];		/* Suffix for these data files */
 	char fflags[GMT_BUFSIZ];	/* Text copy of selected columns */
 	char path[GMT_BUFSIZ];		/* Full path to current data file */
+	char separators[8];		/* List of characters used for column separators */
 	struct X2SYS_DATA_INFO *info;	/* Array of info for each data field */
 };
 
@@ -140,7 +146,7 @@ struct X2SYS_DATA_INFO {
 	bool has_nans;	/* true if there are NaNs in this field */
 	bool do_scale;	/* true if scale != 1 or offset != 0 */
 	char name[32];		/* Name of this data type */
-	char format[32];	/* Output print format for ascii conversion */
+	char format[32];	/* Output print format for ASCII conversion */
 	char intype;		/* Input data type (cuhilfdaA) */
 };
 
@@ -150,7 +156,7 @@ struct X2SYS_FILE_INFO {
 	uint64_t n_rows;	/* Number of rows */
 	uint64_t n_segments;	/* Number of segments in this file */
 	uint64_t *ms_rec;	/* Pointer to array with start record for each segment */
-	char name[32];		/* Name of cruise or agency */
+	char name[64];		/* Name of cruise or agency */
 };
 
 struct X2SYS_BIX {
@@ -214,7 +220,6 @@ struct X2SYS_COE_PAIR {	/* Holds the information for COE between a pair of track
 
 /* Global variables used by x2sys functions */
 
-EXTERN_MSC char *X2SYS_program;
 EXTERN_MSC char *X2SYS_HOME;
 EXTERN_MSC double *x2sys_Y;
 EXTERN_MSC struct MGD77_CONTROL M;
